#!BPY

""" Registration info for Blender menus:
Name: 'Point Cloud demo'
Blender: 240
Group: 'Import'
Tooltip: 'Import a spanning tree for a point cloud'
"""

import Blender, math
from Blender import *
from Blender.Mathutils import *

def filesel_callback(filename):
	scale = 0.02
        divisions = 6
        cverts = []
        for i in range(divisions):
                ang = i * 2 * math.pi / divisions
                cverts.append(Vector(math.cos(ang), math.sin(ang), 0.0))
        for i in range(32):
                ang = i * 2 * math.pi / divisions
                cverts.append(Vector(math.cos(ang), math.sin(ang), 1.0))
        cfaces = []
        for i in range(32):
                j = (i + 1) % divisions
                cfaces.append((i, j, j + divisions, i + divisions))

	scene = Blender.Scene.getCurrent()
        file = open(filename, "r")
	points_obj = Object.New('Mesh', 'PCPoints')
	points = points_obj.getData(False, True)
	edges_obj = Object.New('Mesh', 'PCEdges')
	edges = edges_obj.getData(False, True)
	everts = []
	efaces = []
	emats = []
	xaxis = Vector(1.0, 0.0, 0.0)
	yaxis = Vector(0.0, 1.0, 0.0)
        for line in file:
                coords = [float(x) for x in line.split()]
                if len(coords) == 3:
			points.verts.extend(Vector(coords))
		else:
			p1 = Vector(coords[0:3])
			p2 = Vector(coords[3:6])
			delta = p2 - p1
			if delta.length < scale: continue
			d = Vector(delta)
			d.normalize()
			axis = xaxis
			if AngleBetweenVecs(d, axis) > 0.01: axis = yaxis
			nrm = CrossVecs(d, axis)
			nrm.normalize()
			up = CrossVecs(nrm, d)
			up.normalize()
			transform = Matrix(list(nrm * scale), list(up * scale), list(delta))
			offset = len(edges.verts)
			for v in cverts:
				everts.append(v * transform + p1)
			emats.append(int(coords[6]))
	edges.verts.extend(everts)
	offset = 0
	for i in range(len(everts) / len(cverts)):
		for f in cfaces:
			efaces.append([edges.verts[offset + x] for x in f])
		offset += len(cverts)
	edges.faces.extend(efaces)
	
	preds = [Material.New(name) for name in ['Base', 'Left', 'Right', 'Forward']]
	preds[0].rgbCol = [0.9, 0.9, 0.0]
	preds[1].rgbCol = [0.0, 1.0, 0.0]
	preds[2].rgbCol = [0.0, 0.0, 1.0]
	preds[3].rgbCol = [1.0, 0.0, 0.0]
	edges.materials = preds
	
	for i in range(0, len(efaces), len(cfaces)):
		for j in range(i, i + len(cfaces)):
			edges.faces[j].smooth = 1
			edges.faces[j].mat = emats[i / len(cfaces)]
		
	scene.link(points_obj)
	scene.link(edges_obj)
	World.GetCurrent().setHor([0.0, 0.0, 0.0])
	World.GetCurrent().setMist([0.0, 0.0, 30.0, 0.0])
	Blender.Redraw()

Blender.Window.FileSelector(filesel_callback, "Import PC")
